This is an automatic translation, may be incorrect in some places. See sources and examples!

# Hamming
Library for packing and unpacking data using the Hamming algorithm (redundant data for recovery)
- The order of the algorithm is 4-7 (the higher, the more reliable, but longer processing)
- Recovery of data damaged in transit
- Accepts any type of data

### Compatibility
Compatible with all Arduino platforms (using Arduino functions)

## Content
- [Install](#install)
- [Initialization](#init)
- [Usage](#usage)
- [Example](#example)
- [Versions](#versions)
- [Bugs and feedback](#feedback)

<a id="install"></a>
## Installation
- The library can be found by the name **Hamming** and installed through the library manager in:
    - Arduino IDE
    - Arduino IDE v2
    - PlatformIO
- [Download Library](https://github.com/GyverLibs/Hamming/archive/refs/heads/main.zip) .zip archive for manual installation:
    - Unzip and put in *C:\Program Files (x86)\Arduino\libraries* (Windows x64)
    - Unzip and put in *C:\Program Files\Arduino\libraries* (Windows x32)
    - Unpack and put in *Documents/Arduino/libraries/*
    - (Arduino IDE) automatic installation from .zip: *Sketch/Include library/Add .ZIP library…* and specify the downloaded archive
- Read more detailed instructions for installing libraries [here] (https://alexgyver.ru/arduino-first/#%D0%A3%D1%81%D1%82%D0%B0%D0%BD%D0%BE% D0%B2%D0%BA%D0%B0_%D0%B1%D0%B8%D0%B1%D0%BB%D0%B8%D0%BE%D1%82%D0%B5%D0%BA)<a id="init"></a>
## Initialization
```cpp
Hamming<5> buf; // <> - code order (4-7)
```

<a id="usage"></a>
## Usage
```cpp
bool pack(T&data); // pack data of any type into a buffer
bool pack(uint8_t *ptr, uint32_t size) // pack data from the pointer and size into a buffer
uint32_t unpack(uint8_t* data, uint32_t size); // unpack data. return: 0 OK, 1 fixed bugs, 2 and 3 - have uncorrected bugs
uint8_t status(); // return: 0 OK, 1 fixed errors, 2 and 3 - there are uncorrected errors
uint32_t length(); // buffer size (greater than input data size)
void stop(); // free buffer
uint8_t *buffer; // internal buffer
```

<a id="example"></a>
## Example
See **examples** for other examples!
```cpp
#include <Hamming.h>
void setup() {
  Serial.begin(9600);

  // created a date (any type)
  char data0[] = "Hello, world! Lorem Ipsum";

  // packed
  Hamming<5> buf; // <> - code order (4-7)
  // pack into internal buffer buf.buffer
  buf.pack(data0); // 12ms
  
  // packed data is stored in buf.buffer with size buf.length()
  // you can "send" them
  
  // ======== "TRANSFER" ========

  // fucked up part (two bytes!)
  buf.buffer[5] = 0;
  buf.buffer[6] = 0;
  
  // ======== "RECEPTION" ========

  // unpack. The order in <> is the same!
  Hamming<5> buf2;
  
  // we pass the "received" packed data and their length (it must also be passed or known)
  buf2.unpack(buf.buffer, buf.length()); // 6ms
  
  // output as a string
  Serial.println((char*)buf2.buffer);
  
  // display unpacking status
  Serial.println(buf2.status());
}

void loop() {
}
```

<a id="versions"></a>
## Versions
- v1.0
- v1.1 - fixed critical bug

<a id="feedback"></a>
## Bugs and feedback
When you find bugs, create an **Issue**, or better, immediately write to the mail [alex@alexgyver.ru](mailto:alex@alexgyver.ru)
The library is open for revision and your **Pull Request**'s!